"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.API = void 0;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importDefault(require("axios"));
const HackMDErrors = tslib_1.__importStar(require("./error"));
const defaultOption = {
    unwrapData: true,
};
class API {
    constructor(accessToken, hackmdAPIEndpointURL = "https://api.hackmd.io/v1", options = { wrapResponseErrors: true }) {
        this.accessToken = accessToken;
        this.hackmdAPIEndpointURL = hackmdAPIEndpointURL;
        this.options = options;
        if (!accessToken) {
            throw new HackMDErrors.MissingRequiredArgument('Missing access token when creating HackMD client');
        }
        this.axios = axios_1.default.create({
            baseURL: hackmdAPIEndpointURL,
            headers: {
                "Content-Type": "application/json",
            }
        });
        this.axios.interceptors.request.use((config) => {
            config.headers.Authorization = `Bearer ${accessToken}`;
            return config;
        }, (err) => {
            return Promise.reject(err);
        });
        if (options.wrapResponseErrors) {
            this.axios.interceptors.response.use((response) => {
                return response;
            }, async (err) => {
                if (!err.response) {
                    return Promise.reject(err);
                }
                if (err.response.status >= 500) {
                    throw new HackMDErrors.InternalServerError(`HackMD internal error (${err.response.status} ${err.response.statusText})`, err.response.status, err.response.statusText);
                }
                else if (err.response.status === 429) {
                    throw new HackMDErrors.TooManyRequestsError(`Too many requests (${err.response.status} ${err.response.statusText})`, err.response.status, err.response.statusText, parseInt(err.response.headers['x-ratelimit-userlimit'], 10), parseInt(err.response.headers['x-ratelimit-userremaining'], 10), parseInt(err.response.headers['x-ratelimit-userreset'], 10));
                }
                else {
                    throw new HackMDErrors.HttpResponseError(`Received an error response (${err.response.status} ${err.response.statusText}) from HackMD`, err.response.status, err.response.statusText);
                }
            });
        }
    }
    async getMe(options = defaultOption) {
        return this.unwrapData(this.axios.get("me"), options.unwrapData);
    }
    async getHistory(options = defaultOption) {
        return this.unwrapData(this.axios.get("history"), options.unwrapData);
    }
    async getNoteList(options = defaultOption) {
        return this.unwrapData(this.axios.get("notes"), options.unwrapData);
    }
    async getNote(noteId, options = defaultOption) {
        return this.unwrapData(this.axios.get(`notes/${noteId}`), options.unwrapData);
    }
    async createNote(payload, options = defaultOption) {
        return this.unwrapData(this.axios.post("notes", payload), options.unwrapData);
    }
    async updateNoteContent(noteId, content, options = defaultOption) {
        return this.unwrapData(this.axios.patch(`notes/${noteId}`, { content }), options.unwrapData);
    }
    async updateNote(noteId, payload, options = defaultOption) {
        return this.unwrapData(this.axios.patch(`notes/${noteId}`, payload), options.unwrapData);
    }
    async deleteNote(noteId, options = defaultOption) {
        return this.unwrapData(this.axios.delete(`notes/${noteId}`), options.unwrapData);
    }
    async getTeams(options = defaultOption) {
        return this.unwrapData(this.axios.get("teams"), options.unwrapData);
    }
    async getTeamNotes(teamPath, options = defaultOption) {
        return this.unwrapData(this.axios.get(`teams/${teamPath}/notes`), options.unwrapData);
    }
    async createTeamNote(teamPath, payload, options = defaultOption) {
        return this.unwrapData(this.axios.post(`teams/${teamPath}/notes`, payload), options.unwrapData);
    }
    async updateTeamNoteContent(teamPath, noteId, content) {
        return this.axios.patch(`teams/${teamPath}/notes/${noteId}`, { content });
    }
    async updateTeamNote(teamPath, noteId, options) {
        return this.axios.patch(`teams/${teamPath}/notes/${noteId}`, options);
    }
    async deleteTeamNote(teamPath, noteId) {
        return this.axios.delete(`teams/${teamPath}/notes/${noteId}`);
    }
    unwrapData(reqP, unwrap = true) {
        if (unwrap) {
            return reqP.then(response => response.data);
        }
        else {
            return reqP;
        }
    }
}
exports.API = API;
exports.default = API;
